@@ -1,4 +1,5 @@
 # SPDX-License-Identifier: Apache-2.0
+# SPDX-FileCopyrightText: Copyright contributors to the vLLM project
 
 import time
 from collections.abc import MutableSequence
@@ -7,7 +8,7 @@ from dataclasses import dataclass
 from typing import Any, Generic, Optional, Union
 
 import torch
-from typing_extensions import TypeVar, deprecated
+from typing_extensions import TypeVar
 
 from vllm.logger import init_logger
 from vllm.lora.request import LoRARequest
@@ -76,14 +77,6 @@ class PoolingOutput:
         return (isinstance(other, self.__class__) and bool(
             (self.data == other.data).all()))
 
-    @property
-    @deprecated("`LLM.encode()` now stores raw outputs in the `data` "
-                "attribute. To return embeddings, use `LLM.embed()`. "
-                "To return class probabilities, use `LLM.classify()` "
-                "and access the `probs` attribute. ")
-    def embedding(self) -> list[float]:
-        return self.data.tolist()
-
 
 class RequestOutput:
     """The output data of a completion request to the LLM.
@@ -506,12 +499,6 @@ class ScoringOutput:
     def __repr__(self) -> str:
         return f"ScoringOutput(score={self.score})"
 
-    @property
-    @deprecated("`LLM.score()` now returns scalar scores. "
-                "Please access it via the `score` attribute. ")
-    def embedding(self) -> list[float]:
-        return [self.score]
-
 
 class ScoringRequestOutput(PoolingRequestOutput[ScoringOutput]):
 
