@@ -34,18 +34,19 @@ def _is_fa2_supported(device = None) -> Tuple[bool, Optional[str]]:
         return False, f"FA2 is unavaible due to: {FA2_UNAVAILABLE_REASON}"
     if torch.cuda.get_device_capability(device)[0] < 8:
         return False, \
-            "FA2 is only supported on devices with compute capability < 8"
+            "FA2 is only supported on devices with compute capability >= 8"
     return True, None
     
 def _is_fa3_supported(device = None) -> Tuple[bool, Optional[str]]:
     if not FA3_AVAILABLE:
         return False, f"FA3 is unavaible due to: {FA3_UNAVAILABLE_REASON}"
     if torch.cuda.get_device_capability(device)[0] < 8 \
+        or torch.cuda.get_device_capability(device)[0] >= 10 \
         or torch.cuda.get_device_capability(device) == (8, 6) \
         or torch.cuda.get_device_capability(device) == (8, 9):
         return False, \
             "FA3 is only supported on devices with compute capability >= 8" \
-            " excluding 8.6 and 8.9"
+            " excluding 8.6 and 8.9 and Blackwell archs (>=10)"
     return True, None
 
 def is_fa_version_supported(fa_version: int, device = None) -> bool:
@@ -59,7 +60,7 @@ def fa_version_unsupported_reason(fa_version: int, device = None) \
     -> Optional[str]:
     assert fa_version in [2, 3], f"Unsupported FA version: {fa_version}"
     if fa_version == 2:
-        return _is_fa3_supported(device)[1]
+        return _is_fa2_supported(device)[1]
     elif fa_version == 3:
         return _is_fa3_supported(device)[1]
 
@@ -82,6 +83,7 @@ def flash_attn_varlen_func(
     max_seqlen_k,
     cu_seqlens_k=None, # only used for non-paged prefill
     seqused_k=None,
+    q_v=None,
     dropout_p=0.0,
     softmax_scale=None,
     causal=False,
@@ -91,10 +93,12 @@ def flash_attn_varlen_func(
     deterministic=False,
     return_attn_probs=False,
     block_table=None,
-    *,
     return_softmax_lse=False,
     out=None,
     fa_version: int = DEFAULT_FA_VERSION,
+    q_descale=None,
+    k_descale=None,
+    v_descale=None,
 ):
     """dropout_p should be set to 0.0 during evaluation
     Supports multi-query and grouped-query attention (MQA/GQA) by passing in K, V with fewer heads
@@ -193,9 +197,11 @@ def flash_attn_varlen_func(
             None,
         )
     elif fa_version == 3:
+        assert alibi_slopes is None, "Alibi is not supported in FA3"
         out, softmax_lse, _, _ = torch.ops._vllm_fa3_C.fwd(
             q, k, v,
             None, None,       # k_new, v_new
+            q_v,
             out,
             cu_seqlens_q,
             cu_seqlens_k,     # cu_seqlens_k
@@ -203,10 +209,10 @@ def flash_attn_varlen_func(
             None, seqused_k,  # seqused_q, seqused_k
             max_seqlen_q, max_seqlen_k,
             block_table,
-            alibi_slopes,
             None,             # kv_batch_idx
+            None,             # leftpad_k
             None, None,       # rotary_cos, rotary_sin
-            None, None, None, # q_descale, k_descale, v_descale
+            q_descale, k_descale, v_descale,
             softmax_scale,
             causal,
             real_window_size[0], real_window_size[1],
@@ -245,6 +251,9 @@ def flash_attn_with_kvcache(
     *,
     out=None,
     fa_version: int = DEFAULT_FA_VERSION,
+    q_descale=None,
+    k_descale=None,
+    v_descale=None,
 ):
     """
     If k and v are not None, k_cache and v_cache will be updated *inplace* with the new values from
@@ -344,7 +353,7 @@ def flash_attn_with_kvcache(
         cache_seqlens = maybe_contiguous(cache_seqlens)
     cache_batch_idx = maybe_contiguous(cache_batch_idx)
     block_table = maybe_contiguous(block_table)
-    
+
     if fa_version == 2:
         out, softmax_lse = torch.ops._vllm_fa2_C.fwd_kvcache(
             q, k_cache, v_cache,
@@ -366,28 +375,30 @@ def flash_attn_with_kvcache(
             num_splits,
         )
     elif fa_version == 3:
+        assert alibi_slopes is None, "Alibi is not supported in FA3"
         out, softmax_lse, _, _ = torch.ops._vllm_fa3_C.fwd(
             q, k_cache, v_cache, # q, k, v
-            k, v,             # k_new, v_new
+            k, v,                # k_new, v_new
+            None,                # q_v
             out,
-            None, None,       # cu_seqlens_q, cu_seqlens_k
-            None,             # cu_seqlens_k_new
+            None, None,          # cu_seqlens_q, cu_seqlens_k
+            None,                # cu_seqlens_k_new
             None, cache_seqlens, # seqused_q, seqused_k
-            None, None,       # max_seqlen_q, max_seqlen_k
+            None, None,          # max_seqlen_q, max_seqlen_k
             block_table,
-            alibi_slopes,
-            cache_batch_idx,  # kv_batch_idx
-            None, None,       # rotary_cos, rotary_sin
-            None, None, None, # q_descale, k_descale, v_descale
+            cache_batch_idx,     # kv_batch_idx
+            None,                # leftpad_k
+            None, None,          # rotary_cos, rotary_sin
+            q_descale, k_descale, v_descale,
             softmax_scale,
             causal,
             window_size[0], window_size[1],
-            0,                # sink_token_length
+            0,                   # sink_token_length
             softcap,
-            True,             # rotary_interleaved
-            num_splits,       # num_splits
-            None,             # pack_gqa
-            0,                # sm_margin
+            rotary_interleaved,  # rotary_interleaved
+            num_splits,          # num_splits
+            None,                # pack_gqa
+            0,                   # sm_margin
         )
     else:
         raise ValueError(f"Unsupported FA version: {fa_version}")
@@ -558,4 +569,4 @@ def sparse_attn_varlen_func(
         return_attn_probs and dropout_p > 0,
         None,
     )
-    return (out, softmax_lse) if return_softmax_lse else out
\ No newline at end of file
+    return (out, softmax_lse) if return_softmax_lse else out
